@@ -20,9 +20,9 @@
     pipelka@teleweb.at
  
     Last Update:      $Author: braindead $
-    Update Date:      $Date: 2004/01/22 16:52:42 $
+    Update Date:      $Date: 2004/03/01 20:51:40 $
     Source File:      $Source: /cvsroot/paragui/paragui/include/pgwidget.h,v $
-    CVS/RCS Revision: $Revision: 1.3.6.3.2.4 $
+    CVS/RCS Revision: $Revision: 1.3.6.3.2.10 $
     Status:           $State: Exp $
 */
 
@@ -95,7 +95,7 @@ public:
 
 	This is the constructor for the PG_Widget class (really!)
 	*/
-	PG_Widget(PG_Widget* parent, const PG_Rect& rect);
+	//PG_Widget(PG_Widget* parent, const PG_Rect& rect);
 
 	/**
 	Creates a PG_Widget with an internal object surface
@@ -104,7 +104,7 @@ public:
 	@param rect		initial position for the widget
 	@param bObjectSurface	flag if a surface for the object should be created
 	*/
-	PG_Widget(PG_Widget* parent, const PG_Rect& rect, bool bObjectSurface);
+	PG_Widget(PG_Widget* parent, const PG_Rect& rect = PG_Rect::null, bool bObjectSurface = false);
 
 	/**
 	Destroys a PG_Widget
@@ -130,7 +130,7 @@ public:
 
 	Loads the defined style of a given widgettype and objectname.
 	*/
-	virtual void LoadThemeStyle(const char* widgettype, const char* objectnmae);
+	virtual void LoadThemeStyle(const char* widgettype, const char* objectname);
 
 	/**
 	Start to drag a widget
@@ -160,23 +160,12 @@ public:
 
 	@param x	new x-position (in parent context)
 	@param y	new y-position (in parent context)
+	@param update true - update screen content / false - no screen update
 	@return		function succeeded
 
 	This function moves the widget
 	*/
-	bool MoveWidget(int x, int y);
-
-	/**
-	Move a widget -- OBSOLETE
-
-	@param x	new x-position (in parent context)
-	@param y	new y-position (in parent context)
-	@return		function succeeded
-
-	This function moves the widget.
-	CAUTION: This function will be removed in the final version. Please use MoveWidget(int x, int y)
-	*/
-	bool MoveWindow(int x, int y);
+	bool MoveWidget(int x, int y, bool update = true);
 
 	/**
 	Move and resize widget
@@ -186,41 +175,19 @@ public:
 
 	This function moves and resizes the widget to fit the given rectangle.
 	*/
-	bool MoveWidget(const PG_Rect& r);
-
-	/**
-	Move and resize widget -- OBSOLETE
-
-	@param r	new widget rectangle (client coordinates)
-	@return		function succeeded
-
-	This function moves and resizes the widget to fit the given rectangle.
-	CAUTION: This function will be removed in the final version. Please use MoveWidget(const PG_Rect& r)
-	*/
-	bool MoveWindow(const PG_Rect& r);
-
-	/**
-	Resize a widget -- OBSOLETE
-
-	@param w	new widget-width
-	@param w	new widget-height
-	@return		function succeeded
-
-	This function resizes the widget. This member function will be removed in the final version.
-	Please use SizeWidget().
-	*/
-	virtual bool SizeWindow(Uint16 w, Uint16 h);
+	bool MoveWidget(const PG_Rect& r, bool update = true);
 
 	/**
 	Resize a widget
 
 	@param w	new widget-width
-	@param w	new widget-height
+	@param h	new widget-height
+	@param update true - update widget after resizing / false - do not display changes
 	@return		function succeeded
 
 	This function resizes the widget
 	*/
-	virtual bool SizeWidget(Uint16 w, Uint16 h);
+	virtual bool SizeWidget(Uint16 w, Uint16 h, bool update = true);
 
 	/**
 	Convert a client (widget) coordinate to a screen position
@@ -252,7 +219,7 @@ public:
 
 	@return	SDL_Surface pointer to the screen surface
 	*/
-	SDL_Surface* GetScreenSurface();
+	//SDL_Surface* GetScreenSurface();
 
 	/**
 	Check if the object is visible
@@ -277,8 +244,9 @@ public:
 	/**
 	Process a native PG_ event
 
-	@param event	PG_ event to process
-	@return		true if the event was sucessfully processed
+	@param event PG_ event to process
+	@param bModal set to true if processing takes place in a modal loop
+	@return	true if the event was sucessfully processed
 
 	ProcessEvent asks the widget to process a given event.
 	*/
@@ -519,7 +487,7 @@ public:
 	void ReleaseUserData();
 
 	/**	*/
-	bool RemoveChild(PG_Widget* child);
+	bool virtual RemoveChild(PG_Widget* child);
 
 	/**
 	Sets text
@@ -639,7 +607,7 @@ public:
 	/**
 	Render text inside the widget
 	@param x x-position where the text should appear
-	@param x y-position where the text should appear
+	@param y y-position where the text should appear
 	@param text pointer to text string
 	*/
 	void DrawText(int x, int y, const char* text);
@@ -647,7 +615,7 @@ public:
 	/**
 	Render text inside the widget and clip to a given clipping rectangle
 	@param x x-position where the text should appear
-	@param x y-position where the text should appear
+	@param y y-position where the text should appear
 	@param text pointer to text string
 	@param cliprect text bounding rectangle
 	*/
@@ -657,16 +625,16 @@ public:
 	Render text inside the widget and set the font color
 	@param rect rectangle where the text should appear
 	@param text pointer to text string
-	@param color color of the rendered text
+	@param c color of the rendered text
 	*/
 	void DrawText(const PG_Rect& rect, const char* text, const PG_Color& c);
 
 	/**
 	Render text inside the widget and set the font color
 	@param x x-position where the text should appear
-	@param x y-position where the text should appear
+	@param y y-position where the text should appear
 	@param text pointer to text string
-	@param color color of the rendered text
+	@param c color of the rendered text
 	*/
 	void DrawText(int x, int y, const char* text, const PG_Color& c);
 
@@ -677,7 +645,7 @@ public:
 	Set the transparency of the drawing object
 	@param	t						transparency (0 - opaque / 255 - fully transparent)
 	*/
-	virtual void SetTransparency(Uint8 t);
+	virtual void SetTransparency(Uint8 t, bool bRecursive = false);
 
 	/**
 	Get the transparency of the drawing object
@@ -687,16 +655,13 @@ public:
 
 	/**
 	Set the clipping rectangle for the object
-	@param	x					x-position in client coordinates
-	@param	y					y-position in client coordinates
-	@param	w					width of the clipping rectangle
-	@param	h					height of the clipping rectangle
+	@param r clipping rectangle
 	*/
 	void SetClipRect(PG_Rect& r);
 
 	/**
 	Get the current clipping rectangle
-	@return						the clipping rectangle
+	@return the clipping rectangle
 	*/
 	PG_Rect* GetClipRect();
 
@@ -730,7 +695,7 @@ public:
 	/**
 	Enter modal mode
 	*/
-	virtual int RunModal();
+	virtual void RunModal();
 
 	/**
 	Quit modal mode
@@ -772,7 +737,7 @@ public:
 	void SetHidden(bool hidden);
 	
 	bool IsHidden();
-	
+
 protected:
 
 	/**
@@ -800,7 +765,7 @@ protected:
 	MoveWidget events.
 	CAUTION: This function will be removed in the final version
 	*/
-	virtual void eventMoveWindow(int x, int y);
+	//virtual void eventMoveWindow(int x, int y);
 
 	/**
 	Callback for the SizeWidget event
@@ -819,25 +784,25 @@ protected:
 	SizeWidget events.
 	CAUTION: This function will be removed in the final version
 	*/
-	virtual void eventSizeWindow(Uint16 w, Uint16 h);
+	//virtual void eventSizeWindow(Uint16 w, Uint16 h);
 
 	/**
 	overridable eventhandler to draw the object surface
 	@param surface the widgets drawing surface.
 	@param rect rectangle to draw within.
 	This eventhandler is called whenevener the widget contents should be redrawn.
-	@Note Only widgets with drawing surfaces (see the constructor) will call this eventhandler.
+	@note Only widgets with drawing surfaces (see the constructor) will call this eventhandler.
 	*/
 	virtual void eventDraw(SDL_Surface* surface, const PG_Rect& rect);
 
 	/**
 	overridable eventhandler to blit the widget contents to the screen
-	@param srf surface to be blitted
+	@param surface pointer to SDL_Surface to be blitted
 	@param src source rectangle (client coordinates)
 	@param dst destination rectangle (screen coordinates)
 	Override this eventhandler to perform custom blitting behaviour (without drawing onto the widget surface).
 	*/
-	virtual void eventBlit(SDL_Surface* srf, const PG_Rect& src, const PG_Rect& dst);
+	virtual void eventBlit(SDL_Surface* surface, const PG_Rect& src, const PG_Rect& dst);
 
 	/**
 	overridable eventhandler called whenever the widget gets shown.
@@ -858,6 +823,8 @@ protected:
 	*/
 	virtual bool eventQuitModal(int id, PG_MessageObject* widget, unsigned long data);
 
+	void SetParent(PG_Widget* parent);
+
 	/**  */
 	void FadeOut();
 
@@ -881,7 +848,7 @@ protected:
 	/**
 	pointer to the screen surface
 	*/
-	SDL_Surface* my_srfScreen;
+	//SDL_Surface* my_srfScreen;
 
 	/**
 	text attached to the widget
@@ -895,10 +862,6 @@ protected:
 
 private:
 
-	void InitWidget(PG_Widget* parent, bool bObjectSurface);
-	//void AddChildToCache(PG_Widget *child, const char *name);
-	//void AddChildToCache(PG_Widget *child, int id);
-    
 	PG_Widget(const PG_Widget&);
 	PG_Widget& operator=(const PG_Widget&);
 
@@ -909,7 +872,7 @@ private:
 	// this is a bit rude but neccessary for future binary compatibility
 	// because adding non-static data members would break the ABI.
 	// For this we put all private data into a dynamically created struct.
-	PG_WidgetDataInternal* my_internaldata;
+	PG_WidgetDataInternal* _mid;
 };
 
 #endif // PG_WIDGET_H
