@@ -20,9 +20,9 @@
     pipelka@teleweb.at
  
     Last Update:      $Author: braindead $
-    Update Date:      $Date: 2004/01/31 09:15:53 $
+    Update Date:      $Date: 2004/03/09 09:18:25 $
     Source File:      $Source: /cvsroot/paragui/paragui/src/core/pgapplication.cpp,v $
-    CVS/RCS Revision: $Revision: 1.2.4.22.2.6 $
+    CVS/RCS Revision: $Revision: 1.2.4.22.2.13 $
     Status:           $State: Exp $
 */
 
@@ -63,6 +63,7 @@ bool PG_Application::emergencyQuit = false;
 bool PG_Application::enableBackground = true;
 bool PG_Application::enableAppIdleCalls = false;
 SDL_Surface *PG_Application::my_mouse_pointer = NULL;
+SDL_Surface *PG_Application::my_mouse_backingstore = NULL;
 PG_Rect PG_Application::my_mouse_position = PG_Rect(0,0,0,0);
 PG_Application::CursorMode PG_Application::my_mouse_mode = PG_Application::HARDWARE;
 PG_Font* PG_Application::DefaultFont = NULL;
@@ -122,13 +123,14 @@ PG_Application::PG_Application() {
 }
 
 PG_Application::~PG_Application() {
+	// shutdown log (before deleting all the widgets)
+	PG_LogConsole::Done();
+	
+	// remove remaining widgets
 	Shutdown();
 	
 	pGlobalApp = NULL;
 
-	// shutdown log
-	PG_LogConsole::Done();
-	
 	// remove all archives from PG_FileArchive
 	PG_FileArchive::RemoveAllArchives();
 }
@@ -205,6 +207,8 @@ int PG_Application::RunEventLoop(void* data) {
 		while(SDL_PeepEvents(&event, 1, SDL_GETEVENT, SDL_MOUSEMOTIONMASK) > 0)
 			;
 		
+		ClearOldMousePosition();
+
 		if(enableAppIdleCalls) {
 			if (SDL_PollEvent(&event) == 0) {
 				object->eventIdle();
@@ -227,23 +231,18 @@ void PG_Application::ClearOldMousePosition() {
 		return;
 	}
 
-	PG_RectList *widgetList = PG_Widget::GetWidgetList();
-	PG_Application::RedrawBackground(my_mouse_position);
+	if(GetBulkMode() || my_mouse_backingstore==NULL) {
+		return;
+	}
 
-//widgetlist::Blit() already performs all the other operations done here,
-//so one can safely remove the rest of the function calls
-//	SDL_SetClipRect(screen, &my_mouse_position);
-//	PG_RectList backlist = widgetList->Intersect(&my_mouse_position);
-//	backlist.Blit(my_mouse_position);
-//	SDL_SetClipRect(screen, 0);
-    widgetList->Blit(my_mouse_position);
+	SDL_BlitSurface(my_mouse_backingstore, NULL, GetScreen(), &my_mouse_position);
     
 	return;
 }
 
-void PG_Application::DrawCursor() {
+void PG_Application::DrawCursor(bool update) {
 	int x, y;
-	PG_Rect saved;
+
 	if(!my_mouse_pointer || my_mouse_mode != SOFTWARE) {
 		return;
 	}
@@ -251,18 +250,72 @@ void PG_Application::DrawCursor() {
 		// Hide hardware cursor if visible
 		SDL_ShowCursor(SDL_DISABLE);
 	}
-	//SDL_PumpEvents();
+
 	SDL_GetMouseState(&x, &y);
-	saved = my_mouse_position;
-	if(!GetBulkMode()) {
-		ClearOldMousePosition();
+
+	Sint16 dx = x - my_mouse_position.my_xpos;
+	Sint16 dy = y - my_mouse_position.my_ypos;
+
+	// calculate vertical update rect
+	PG_Rect vertical(0, my_mouse_position.my_ypos, abs(dx), my_mouse_pointer->h + abs(dy));
+	if(dx >= 0) {
+		vertical.my_xpos = my_mouse_position.my_xpos;
+	} else {
+		vertical.my_xpos = my_mouse_position.my_xpos + my_mouse_pointer->w + dx;
+	}	
+
+	// calculate horizontal update rect
+	PG_Rect horizontal(my_mouse_position.my_xpos, 0, my_mouse_pointer->w + abs(dx), abs(dy));	
+	if(dy >= 0) {
+		horizontal.my_ypos = my_mouse_position.my_ypos;
+	} else {
+		horizontal.my_ypos = my_mouse_position.my_ypos + my_mouse_pointer->h + dy;
 	}
+	
+	// clipping
+	if(vertical.my_xpos + vertical.my_width > screen->w) {
+		if(vertical.my_xpos >= screen->w) {
+			vertical.my_xpos = screen->w - 1;
+		}
+		vertical.my_width = screen->w - vertical.my_xpos;
+	}
+	if(vertical.my_ypos + vertical.my_height > screen->h) {
+		if(vertical.my_ypos >= screen->h) {
+			vertical.my_ypos = screen->h - 1;
+		}
+		vertical.my_height = screen->h - vertical.my_ypos;
+	}
+
+	if(horizontal.my_xpos + horizontal.my_width > screen->w) {
+		if(horizontal.my_xpos >= screen->w) {
+			horizontal.my_xpos = screen->w - 1;
+		}
+		horizontal.my_width = screen->w - horizontal.my_xpos;
+	}
+	if(horizontal.my_ypos + horizontal.my_height > screen->h) {
+		if(horizontal.my_ypos >= screen->h) {
+			horizontal.my_ypos = screen->h - 1;
+		}
+		horizontal.my_height = screen->h - horizontal.my_ypos;
+	}		
+
 	my_mouse_position.my_xpos = x;
 	my_mouse_position.my_ypos = y;
+	my_mouse_position.my_width = my_mouse_pointer->w;
+	my_mouse_position.my_height = my_mouse_pointer->h;
+	
+	// backup current cursor area
+	if(my_mouse_backingstore == NULL) {
+		my_mouse_backingstore = PG_Draw::CreateRGBSurface(my_mouse_pointer->w, my_mouse_pointer->h);
+	}
+	SDL_BlitSurface(GetScreen(), &my_mouse_position, my_mouse_backingstore, NULL);
+	
+	// draw cursor
 	SDL_BlitSurface(my_mouse_pointer, 0, screen, &my_mouse_position);
-	if(!GetBulkMode()) {
-		SDL_UpdateRects(screen, 1, &saved);
-		SDL_UpdateRects(screen, 1, &my_mouse_position);
+
+	if(!GetBulkMode() && update) {
+		SDL_Rect rects[3] = {horizontal, vertical, my_mouse_position};
+		SDL_UpdateRects(screen, 3, rects);
 	}
 }
 void PG_Application::Quit() {
@@ -318,8 +371,10 @@ void PG_Application::SetCursor(SDL_Surface *image) {
 		if(!my_mouse_pointer) {
 			return;
 		}
+		UnloadSurface(my_mouse_backingstore);
+		my_mouse_backingstore = NULL;
 		UnloadSurface(my_mouse_pointer);
-		my_mouse_pointer = 0;
+		my_mouse_pointer = NULL;
 		ClearOldMousePosition();
 		SDL_UpdateRects(screen, 1, &my_mouse_position);
 		SDL_ShowCursor(SDL_ENABLE);
@@ -331,6 +386,9 @@ void PG_Application::SetCursor(SDL_Surface *image) {
 		UnloadSurface(my_mouse_pointer);
 		my_mouse_pointer = image;
 	}
+
+	UnloadSurface(my_mouse_backingstore);
+	my_mouse_backingstore = NULL;
 	image->refcount++;
 	DrawCursor();
 }
@@ -399,6 +457,7 @@ bool PG_Application::SetBackground(const char* filename, int mode) {
 		my_freeBackground = true;
 		my_backmode = mode;
 		RedrawBackground(PG_Rect(0,0,screen->w,screen->h));
+		PG_Widget::GetWidgetList()->Blit();
 		return true;
 	} else {
 		PG_LogWRN("Failed to load '%s'",(char*)filename);
@@ -426,20 +485,19 @@ bool PG_Application::SetBackground(SDL_Surface* surface, int mode) {
 	my_backmode = mode;
 
 	RedrawBackground(PG_Rect(0,0,screen->w,screen->h));
+	PG_Widget::GetWidgetList()->Blit();
 	return true;
 }
 
 /**  */
 void PG_Application::RedrawBackground(const PG_Rect& rect) {
-	static PG_Rect screenrect(0,0,screen->w,screen->h);
-
-	if(GetBulkMode() || !enableBackground) {
+	if(GetBulkMode()) {
 		return;
 	}
 
 	PG_Rect fillrect = rect;
 
-	if(!my_background) {
+	if(!my_background || !enableBackground) {
 		SDL_FillRect(screen, (SDL_Rect*)&fillrect, my_backcolor.MapRGB(screen->format));
 		return;
 	}
@@ -449,21 +507,25 @@ void PG_Application::RedrawBackground(const PG_Rect& rect) {
 		if(my_scaled_background && 
 		   (my_scaled_background->w != screen->w ||
 		    my_scaled_background->h != screen->h)) {
-			SDL_FreeSurface(my_scaled_background); // size mismatch
-			my_scaled_background = 0;
+			UnloadSurface(my_scaled_background); // size mismatch
+			my_scaled_background = NULL;
 		}
 		if(!my_scaled_background) {
-			my_scaled_background =
-				PG_Draw::ScaleSurface(my_background,
-						      static_cast<Uint16>(screen->w), static_cast<Uint16>(screen->h));
+			SDL_Surface* temp = PG_Draw::ScaleSurface(my_background, static_cast<Uint16>(screen->w), static_cast<Uint16>(screen->h));
+			my_scaled_background = SDL_DisplayFormat(temp);
+			UnloadSurface(temp);
+			/*PG_Draw::ScaleSurface(my_background,
+						      static_cast<Uint16>(screen->w), static_cast<Uint16>(screen->h));*/
 		}
 		SDL_GetClipRect(screen, (SDL_Rect*)&fillrect);
 		SDL_SetClipRect(screen, (SDL_Rect*)&rect);
+		SDL_SetAlpha(my_scaled_background, 0, 0);
 		SDL_BlitSurface(my_scaled_background, (SDL_Rect*)&rect, screen, (SDL_Rect*)&rect);
 		SDL_SetClipRect(screen, (SDL_Rect*)&fillrect);
 
 	} else {
-		PG_Draw::DrawTile(screen, screenrect, rect, my_background);
+		SDL_SetAlpha(my_background, 0, 0);
+		PG_Draw::DrawTile(screen, PG_Rect(0,0,screen->w,screen->h), rect, my_background);
 	}
 }
 
@@ -707,6 +769,10 @@ void PG_Application::Shutdown() {
 	// delete the default font
 	delete DefaultFont;
 	DefaultFont = NULL;
+
+	// remove cursor backing store	
+	UnloadSurface(my_mouse_backingstore);
+	my_mouse_backingstore = NULL;
 }
 
 void PG_Application::SetEmergencyQuit(bool esc) {
@@ -773,17 +839,17 @@ void PG_Application::SetIcon(const char *filename) {
 
 	//Clean up
 	delete[] mask;
-	return;
 }
 
 void PG_Application::SetCaption(const char *title, const char *icon) {
-	SDL_WM_SetCaption(title,icon);
-	return;
+	SDL_WM_SetCaption(title, NULL);
+	if (icon != NULL) {
+		SetIcon(icon);
+	}
 }
 
 void PG_Application::GetCaption(char **title, char **icon) {
 	SDL_WM_GetCaption(title,icon);
-	return;
 }
 
 int PG_Application::Iconify(void) {
@@ -813,24 +879,24 @@ static PG_Widget *FindInChildObjects(PG_RectList *RectList, const char *Name) {
 	if (!Name)
             return NULL;
     
-	PG_RectList::iterator list = RectList->begin();
+	PG_Widget* list = RectList->first();
 
-	while(list != RectList->end()) {
-		if (strcmp((*list)->GetName(), Name) == 0) {
-			return((*list));
+	while(list != NULL) {
+		if (strcmp(list->GetName(), Name) == 0) {
+			return list;
 		}
 
-		PG_Widget* result = (*list)->FindChild(Name);
+		PG_Widget* result = list->FindChild(Name);
 		if(result != NULL) {
 			return result;
 		}
 
-		retWidget = FindInChildObjects((*list)->GetChildList(), Name);
+		retWidget = FindInChildObjects(list->GetChildList(), Name);
 		if (retWidget != NULL) {
 			return retWidget;
 		}
 
-		list++;
+		list = list->next();
 	}
 
 	return NULL;
@@ -846,25 +912,25 @@ static inline PG_Widget *FindInChildObjects(PG_RectList *RectList, int id) {
 	if (id < 0)
             return 0;
     
-	PG_RectList::iterator list = RectList->begin();
+	PG_Widget* list = RectList->first();
 
-	while(list != RectList->end()) {
-		if ((*list)->GetID() == id) {
-			return((*list));
+	while(list != NULL) {
+		if (list->GetID() == id) {
+			return list;
 		}
 
 
-		PG_Widget* result = (*list)->FindChild(id);
+		PG_Widget* result = list->FindChild(id);
 		if(result != NULL) {
 			return result;
 		}
         
-		retWidget = FindInChildObjects((*list)->GetChildList(), id);
+		retWidget = FindInChildObjects(list->GetChildList(), id);
 		if (retWidget != NULL) {
 			return retWidget;
 		}
 
-		list++;
+		list = list->next();
 	}
 
 	return NULL;
@@ -902,9 +968,9 @@ void PG_Application::SetFontName(const char *Name) {
 	DefaultFont->SetName(Name);
 }
 
-SDL_Surface* PG_Application::GetScreen() {
+/*SDL_Surface* PG_Application::GetScreen() {
 	return screen;
-}
+}*/
 
 int PG_Application::GetScreenHeight() {
 	return screen->h;
@@ -980,9 +1046,14 @@ void PG_Application::FlushEventQueue() {
 	}
 }
 
+#ifdef WIN32
+SDL_Surface* PG_Application::GetScreen() {
+	return screen;
+}
+#endif
+
 /*
  * Local Variables:
  * c-basic-offset: 8
  * End:
  */
-
