@@ -24,6 +24,7 @@
 #include "pglog.h"
 #include "pgpopupmenu.h"
 #include "pgmenubar.h"
+#include "pgscrollwidget.h"
 
 #include <cstring>
 #include <stdarg.h>
@@ -207,6 +208,14 @@ int PG_Layout::GetParamIMode(const char **Source, char *What) {
 		ret = BKMODE_TILE;
 	if (tcscmp(T(c),T("stretch")) == 0)
 		ret = BKMODE_STRETCH;
+	if (tcscmp(T(c),T("stretch")) == 0)
+		ret = BKMODE_STRETCH;
+	if (tcscmp(T(c),T("3tileh")) == 0)
+		ret = BKMODE_3TILEH;
+	if (tcscmp(T(c),T("3tilev")) == 0)
+		ret = BKMODE_3TILEV;
+	if (tcscmp(T(c),T("9tile")) == 0)
+		ret = BKMODE_9TILE;
 
 	return(ret);
 }
@@ -566,6 +575,14 @@ static int SetWidgetListAtts(PG_WidgetList *Widget, const char **atts, ParseUser
 	return (INHTAGFLAG_ADDWIDGET | SetThemeWidgetAtts(Widget, atts, XMLParser));
 }
 
+static int SetScrollWidgetAtts(PG_ScrollWidget*Widget, const char **atts, ParseUserData_t *XMLParser) {
+	if (PG_Layout::GetParamInt(atts, "scrollbar") == 1)
+		Widget->EnableScrollBar(1);
+
+	XMLParser->EndTagFlags |= ENDTAGFLAG_WIDGETLIST;
+	return (INHTAGFLAG_ADDWIDGET | SetThemeWidgetAtts(Widget, atts, XMLParser));
+}
+
 static int SetListBoxAtts(PG_ListBox *Widget, const char **atts, ParseUserData_t *XMLParser) {
 	Widget->SetMultiSelect(PG_Layout::GetParamInt(atts, "multisel") == 1);
 
@@ -595,10 +612,10 @@ static int SetScrollBarAtts(PG_ScrollBar *Widget, const char **atts, ParseUserDa
 		Widget->SetPosition(i);
 	}
 
-	i = PG_Layout::GetParamInt(atts, "wsize");
+	/*i = PG_Layout::GetParamInt(atts, "wsize");
 	if (i != -1) {
 		Widget->SetWindowSize(i);
-	}
+	}*/
 
 	i = PG_Layout::GetParamInt(atts, "lsize");
 	if (i != -1) {
@@ -624,7 +641,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 	ParseUserData_t	*XMLParser = (ParseUserData_t	*)userData;
 
 	SaveUserData(XMLParser);
-	PG_Widget* parent = ((XMLParser->EndTagFlags & INHTAGFLAG_ADDWIDGET) == INHTAGFLAG_ADDWIDGET) ? NULL : XMLParser->ParentObject;
+	PG_Widget* parent = /*((XMLParser->EndTagFlags & INHTAGFLAG_ADDWIDGET) == INHTAGFLAG_ADDWIDGET) ? NULL :*/ XMLParser->ParentObject;
 
 	//Tag <layout> <LA>
 	if (IsTag("layout","LA",XML_SECTION_DOC)) {
@@ -666,7 +683,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 			return;
 		}
 
-		PG_DropDown *Widget = new  PG_DropDown (parent, 0, Rect);
+		PG_DropDown *Widget = new PG_DropDown (parent, Rect);
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |=SetDropDownAtts(Widget, atts, XMLParser);
@@ -686,11 +703,13 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		XMLParser->Section = XML_SECTION_BUTTON | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
 		PG_Layout::GetParamRect(atts, "pos", Rect, parent);
 
+		PG_LogDBG("layout: parent = %p", parent);
 		PG_Button	*Widget = new PG_Button(
 		                        parent,
-		                        0,
 		                        Rect,
-		                        PG_Layout::GetParamStr(atts, "text"));
+		                        PG_Layout::GetParamStr(atts, "text"),
+								PG_Layout::GetParamInt(atts, "id"),
+								PG_Layout::GetParamStr(atts, "style"));
 
 		XMLParser->ParentObject = Widget;
 
@@ -703,7 +722,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		XMLParser->Section = XML_SECTION_LABEL | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
 		PG_Layout::GetParamRect(atts, "pos", Rect, parent);
 
-		PG_Label	*Widget = new PG_Label(parent, Rect, "");
+		PG_Label	*Widget = new PG_Label(parent, Rect, "", PG_Layout::GetParamStr(atts, "style"));
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |=SetLabelAtts(Widget, atts, XMLParser);
@@ -841,18 +860,6 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		return;
 	}
 
-	//Tag <widgetlistex> <WX>
-	if (IsTag("widgetlistex","WX",XML_SECTION_BODY)) {
-		XMLParser->Section = XML_SECTION_WIDGETLISTEX | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
-		PG_Layout::GetParamRect(atts, "pos", Rect, parent);
-
-		PG_WidgetListEx *Widget = new PG_WidgetListEx(parent, Rect);
-		XMLParser->ParentObject = Widget;
-
-		XMLParser->InhTagFlags |= SetWidgetListAtts(Widget, atts, XMLParser);
-		return;
-	}
-
 	//Tag <listbox> <LB>
 	if (IsTag("listbox","LB",XML_SECTION_BODY)) {
 		XMLParser->Section = XML_SECTION_LISTBOX | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
@@ -874,8 +881,9 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 			h = 25;
 		}
 
-		PG_ListBoxItem *Widget = new PG_ListBoxItem(h, PG_Layout::GetParamStr(atts, "ltext"));
-		((PG_ListBox *)XMLParser->ParentObject)->AddItem(Widget);
+		// to be updated!
+		PG_ListBoxItem *Widget = new PG_ListBoxItem(parent, h, PG_Layout::GetParamStr(atts, "ltext"));
+		//((PG_ListBox *)XMLParser->ParentObject)->AddItem(Widget);
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |= SetListBoxItemAtts(Widget, atts, XMLParser);
@@ -886,8 +894,9 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 	if (IsTag("columnitem","CI",XML_SECTION_LISTBOX)) {
 		XMLParser->Section = XML_SECTION_COLUMNITEM | XML_SECTION_COMWIDPARAMS;
 
-		PG_ColumnItem *Widget = new PG_ColumnItem(PG_Layout::GetParamInt(atts, "columns"),PG_Layout::GetParamInt(atts, "height"));
-		((PG_ListBox *)XMLParser->ParentObject)->AddItem(Widget);
+		// to be updated!
+		PG_ColumnItem *Widget = new PG_ColumnItem(parent, PG_Layout::GetParamInt(atts, "columns"),PG_Layout::GetParamInt(atts, "height"));
+		//((PG_ListBox *)XMLParser->ParentObject)->AddItem(Widget);
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |= SetListBoxItemAtts(Widget, atts, XMLParser);
@@ -926,10 +935,10 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 
 		PG_RadioButton *Widget = new PG_RadioButton(
 		                             parent,
-		                             0,
 		                             Rect,
 		                             (const char*)PG_Layout::GetParamStr(atts, "text"),
-		                             (PG_RadioButton *)PG_Application::GetWidgetByName(PG_Layout::GetParamStr(atts, "group")));
+		                             (PG_RadioButton *)PG_Application::GetWidgetByName(PG_Layout::GetParamStr(atts, "group")),
+									 0);
 
 		XMLParser->ParentObject = Widget;
 
@@ -944,9 +953,9 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 
 		PG_CheckButton *Widget = new PG_CheckButton(
 		                             parent,
-		                             0,
 		                             Rect,
-		                             PG_Layout::GetParamStr(atts, "text"));
+		                             PG_Layout::GetParamStr(atts, "text"),
+									 0);
 
 		XMLParser->ParentObject = Widget;
 
@@ -962,7 +971,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		PG_ScrollBar::ScrollDirection d = PG_Layout::GetParamScrollDirection(atts, "dir");
 		(d <= 0) ? d = PG_ScrollBar::HORIZONTAL : d = PG_ScrollBar::VERTICAL;
 
-		PG_ScrollBar *Widget = new PG_ScrollBar(parent, 0, Rect, d);
+		PG_ScrollBar *Widget = new PG_ScrollBar(parent, Rect, d);
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |= SetScrollBarAtts(Widget, atts, XMLParser);
@@ -974,7 +983,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		XMLParser->Section = XML_SECTION_SCROLLBAR | XML_SECTION_BODY | XML_SECTION_COMWIDPARAMS;
 		PG_Layout::GetParamRect(atts, "pos", Rect, parent);
 
-		PG_Slider *Widget = new PG_Slider(parent, 0, Rect, PG_Layout::GetParamScrollDirection(atts, "dir"));
+		PG_Slider *Widget = new PG_Slider(parent, Rect, PG_Layout::GetParamScrollDirection(atts, "dir"));
 		XMLParser->ParentObject = Widget;
 
 		XMLParser->InhTagFlags |=SetScrollBarAtts(Widget, atts, XMLParser);
@@ -1007,7 +1016,7 @@ static void XMLStartDoc(void *userData, const char *name, const char **atts) {
 		PG_RichEdit *Widget = new PG_RichEdit(parent, Rect, PG_Layout::GetParamInt(atts, "vresize") == 1 , PG_Layout::GetParamInt(atts, "linewidth"));
 		XMLParser->ParentObject = Widget;
 
-		XMLParser->InhTagFlags |= SetWidgetListAtts(Widget, atts, XMLParser);
+		XMLParser->InhTagFlags |= SetScrollWidgetAtts(Widget, atts, XMLParser);
 		return;
 	}
 
@@ -1123,12 +1132,14 @@ static void XMLEndDoc(void *userData, const char *name) {
 
 	if ((XMLParser->EndTagFlags & ENDTAGFLAG_OBJECT) != 0) {
 		if (((XMLParser->InhTagFlags & INHTAGFLAG_ADDWIDGET) != 0) && ((XMLParser->EndTagFlags & ENDTAGFLAG_WIDGETLIST) == 0)) {
-			WidgetToAdd = XMLParser->ParentObject;
+			//WidgetToAdd = XMLParser->ParentObject;
 //			goto object_end;
 		}
 
 		if ((XMLParser->InhTagFlags & INHTAGFLAG_HIDE) == 0) {
-			XMLParser->ParentObject->Show();
+			if(XMLParser->ParentObject->GetParent() == NULL) {
+				XMLParser->ParentObject->Show();
+			}
 		}
 		else {
 			XMLParser->ParentObject->Hide();
@@ -1144,7 +1155,7 @@ static void XMLEndDoc(void *userData, const char *name) {
 		return;
 	}
 
-	((PG_WidgetList *)(XMLParser->ParentObject))->AddWidget(WidgetToAdd);
+	((PG_WidgetList *)(XMLParser->ParentObject))->AddChild(WidgetToAdd);
 }
 
 static void XMLTextDoc(void *userData, const XML_Char *s, int len) {
