@@ -52,6 +52,12 @@ bool handle_exit(PG_Pointer clientdata) {
 	return true;
 }
 
+bool hande_slideIndent(PG_ScrollBar* s, long pos, PG_Pointer data)  {
+	PG_DropDown* drop = static_cast<PG_DropDown*>(data);
+	drop->SetIndent(pos);
+	return true;
+}
+
 // A new widget with a callback member
 
 class MySliderLabel : public PG_Label {
@@ -84,11 +90,11 @@ public:
 	/** callback handler in another flavor */
 	bool handler_slider_btntrans(long data) {
 		// set transparency of passed in button
-		b->SetTransparency(data, data, data);
+		b->SetTransparency(static_cast<Uint8>(data));
 		b->Update();
 
 		// set transparency of class member (progress)
-		progress->SetTransparency(data);
+		progress->SetTransparency(static_cast<Uint8>(data));
 		progress->Update();
 		return true;
 	}
@@ -118,28 +124,24 @@ PG_Window(parent, r, windowtext, DEFAULT)
 	WidgetList->EnableScrollBar(true, PG_ScrollBar::VERTICAL);
 	WidgetList->EnableScrollBar(true, PG_ScrollBar::HORIZONTAL);
 		
-	(new PG_Button(this, 100, PG_Rect(260,130,110,30), "<< ADD"))->sigClick.connect(slot(*this, &TestWindow::handleButtonClick));
+	(new PG_Button(this, PG_Rect(260,130,110,30), "<< ADD", 100))->sigClick.connect(slot(*this, &TestWindow::handleButtonClick));
 	
-	(new PG_Button(this, 101, PG_Rect(260,165,110,30), ">> REMOVE"))->sigClick.connect(slot(*this, &TestWindow::handleButtonClick));
+	(new PG_Button(this, PG_Rect(260,165,110,30), ">> REMOVE", 101))->sigClick.connect(slot(*this, &TestWindow::handleButtonClick));
 	
-	b = new PG_Button(NULL, BTN_ID_YES, PG_Rect(0,0, 400,50), "YES");
+	b = new PG_Button(WidgetList, PG_Rect(0,0, 400,50), "YES", PG_Button::YES);
 	b->SetTransparency(128,128,128);
 	b->SetFontName("qnx/font.ttf");
-	WidgetList->AddWidget(b);
 
-	PG_Slider* s = new PG_Slider(NULL, 20, PG_Rect(0, 0, 200,20), PG_ScrollBar::HORIZONTAL);
+	PG_Slider* s = new PG_Slider(WidgetList, PG_Rect(0, 0, 200,20), PG_ScrollBar::HORIZONTAL);
 	s->SetRange(0,255);
 	s->SetTransparency(200);
 	s->SetPosition(50);
 
 	s->sigSlide.connect(slot(*this, &TestWindow::handler_slider_btntrans));
+	s->sigSlideEnd.connect(slot(*this, &TestWindow::handler_slider_btntrans));
 
-	WidgetList->AddWidget(s);
-		
-	WidgetList->AddWidget(new PG_LineEdit(NULL, PG_Rect(0,0,80,30)));
-
-	PG_CheckButton* check = new PG_CheckButton(NULL, 10, PG_Rect(0,0,200,25), "CheckButton 2");
-	WidgetList->AddWidget(check);
+	new PG_LineEdit(WidgetList, PG_Rect(0,0,80,30));
+	new PG_CheckButton(WidgetList, PG_Rect(0,0,200,25), "CheckButton 2");
 
 	progress = new PG_ProgressBar(this, PG_Rect(260, 90, 150, 25));
 	progress->SetTransparency(128);
@@ -147,18 +149,21 @@ PG_Window(parent, r, windowtext, DEFAULT)
 	progress->SetID(1001);
 	progress->SetFontAlpha(128);
 		
-	PG_ScrollBar* scroll = new PG_ScrollBar(this, 1, PG_Rect(415,90,20,150));
+	PG_ScrollBar* scroll = new PG_ScrollBar(this, PG_Rect(415,90,20,150));
+	scroll->SetID(1);
 	scroll->sigScrollPos.connect(slot(*this, &TestWindow::handleScrollPos));
 	scroll->sigScrollTrack.connect(slot(*this, &TestWindow::handleScrollPos));
 	scroll->SetRange(0, 100);
 
-	PG_ScrollBar* scroll1 = new PG_ScrollBar(this, 2, PG_Rect(435,90,20,150));
+	PG_ScrollBar* scroll1 = new PG_ScrollBar(this, PG_Rect(435,90,20,150));
+	scroll1->SetID(2);
 	scroll1->sigScrollPos.connect(slot(*this, &TestWindow::handleScrollPos));
 	scroll1->sigScrollTrack.connect(slot(*this, &TestWindow::handleScrollPos));
 	scroll1->SetRange(0, 255);
 
-	PG_DropDown* drop = new PG_DropDown(this, 15, PG_Rect(260, 60, 200,25));
+	PG_DropDown* drop = new PG_DropDown(this, PG_Rect(260, 60, 200,25));
 	drop->SetIndent(5);
+	drop->SetAlignment(PG_Label::CENTER);
 	drop->AddItem("Under construction");
 	drop->AddItem("Item 1");
 	drop->AddItem("Item 2");
@@ -166,7 +171,7 @@ PG_Window(parent, r, windowtext, DEFAULT)
 	drop->AddItem("Item 4");
 	drop->AddItem("Item 5");
 	drop->AddItem("Item 6");
-	drop->Hide();
+	//drop->Hide();
 }
 
 bool TestWindow::handleScrollPos(PG_ScrollBar* widget, long data){
@@ -204,22 +209,20 @@ bool TestWindow::handleScrollTrack(PG_ScrollBar* widget, long data) {
 }
 
 bool TestWindow::handleButtonClick(PG_Button* button) {
-	int label=0;
+	static int label=0;
 	int id = button->GetID();
 
 	if(id == 100) {
-		PG_Label* l = new PG_Label(NULL, PG_Rect(0,0,220,25), "");
+		PG_Label* l = new PG_Label(WidgetList, PG_Rect(0,0,220,25), "");
 		l->SetAlignment(PG_Label::CENTER);
 		l->SetTextFormat("Label %i", ++label);
-		WidgetList->AddWidget(l);
-	
 		return true;
 	}
 
 	if(id == 101) {
 		PG_Widget* w = WidgetList->FindWidget(4);
+		PG_LogDBG("FindWidget(4) = %p", w);
 		if(w != NULL) {
-			WidgetList->RemoveWidget(w, true, true);
 			delete w;
 		}
 		
@@ -254,18 +257,18 @@ void PrintChildObjects(PG_RectList *RectList, char *TabSpace) {
 		return;
 	}
 
-    PG_RectList::iterator list = RectList->begin();
+    PG_Widget* list = RectList->first();
 	const char* name = NULL;
 
-    while(list != RectList->end()) {
-		name = (*list)->GetName();
+    while(list != NULL) {
+		name = list->GetName();
 
 		PG_LogMSG("%s %s",TabSpace,name);
 
 		sprintf(tab,"  %s",TabSpace);	
-		PrintChildObjects((*list)->GetChildList(), tab);
+		PrintChildObjects(list->GetChildList(), tab);
 
-		list++;
+		list = list->next();
 	}
     
 }
@@ -284,7 +287,7 @@ int main(int argc, char* argv[]) {
 	bool bTestMode = false;
 
 	strcpy(theme, "default");
-	
+
 	// initial flags for screensurface
 	Uint32 flags = SDL_SWSURFACE;
 	int bpp = 0;
@@ -293,7 +296,6 @@ int main(int argc, char* argv[]) {
 	
 	// construct the application object
 	PG_Application app;
-	//app.DisableDirtyUpdates(true);
 		
 	for(int c=1; c<argc; c++) {
 
@@ -328,7 +330,7 @@ int main(int argc, char* argv[]) {
 	}
 
 	Uint32 start_ticks = SDL_GetTicks();
-	
+
 	if(!app.LoadTheme(theme)) {
 	    PG_LogERR("Unable to load theme \"%s\"", theme);
 	    return -1;
@@ -336,13 +338,13 @@ int main(int argc, char* argv[]) {
 
 	PG_LogMSG("Loaded theme in %i ms", SDL_GetTicks() - start_ticks);
 
-	if(!app.InitScreen(resx, resy, bpp, flags)){
-		printf("Resolution %dx%d not supported\n", resx, resy);
-		exit(-1);
-	}
-
-	PG_LogMSG("screen initialized after %i ms", SDL_GetTicks() - start_ticks);
-
+        if(!app.InitScreen(resx, resy, bpp, flags)){
+                printf("Resolution %dx%d not supported\n", resx, resy);
+                exit(-1);
+        }
+                                                                                                                    
+        PG_LogMSG("screen initialized after %i ms", SDL_GetTicks() - start_ticks);
+	
 	app.SetCursor(app.GetTheme()->FindSurface("Pointer", "Pointer", "normal"));
 
 	//Splash();
@@ -365,19 +367,19 @@ int main(int argc, char* argv[]) {
 	wnd1.SetID(101);
 	
 	// create 2 radiobutton groups
-	PG_RadioButton radio1(NULL, 1, PG_Rect(50,0,200,25), "RadioButton 1");
-	PG_RadioButton radio2(NULL, 2, PG_Rect(50,25,200,25), "RadioButton 2", &radio1);
-	PG_RadioButton radio3(NULL, 3, PG_Rect(50,50,200,25), "RadioButton 3", &radio1);
+	PG_RadioButton radio1(NULL, PG_Rect(50,0,200,25), "RadioButton 1");
+	PG_RadioButton radio2(NULL, PG_Rect(50,25,200,25), "RadioButton 2", &radio1);
+	PG_RadioButton radio3(NULL, PG_Rect(50,50,200,25), "RadioButton 3", &radio1);
 	radio1.SetAlignment(PG_Label::RIGHT);
 	
-	PG_RadioButton radio4(NULL, 4, PG_Rect(50,90,200,25), "RadioButton 4");
-	PG_RadioButton radio5(NULL, 5, PG_Rect(50,115,200,25), "RadioButton 5", &radio4);
-	PG_RadioButton radio6(NULL, 6, PG_Rect(50,140,200,25), "RadioButton 6", &radio4);
+	PG_RadioButton radio4(NULL, PG_Rect(50,90,200,25), "RadioButton 4");
+	PG_RadioButton radio5(NULL, PG_Rect(50,115,200,25), "RadioButton 5", &radio4);
+	PG_RadioButton radio6(NULL, PG_Rect(50,140,200,25), "RadioButton 6", &radio4);
 
-	PG_CheckButton check1(NULL, 7, PG_Rect(300,140,180,25), "CheckButton 1");
-	PG_PopupMenu   popmenu(NULL, 425, 140, "My Menu");
-	PG_PopupMenu   submenu(NULL, 425, 140, "My SubMenu");
-	PG_PopupMenu   subsubmenu(NULL, 425, 140, "");
+	PG_CheckButton check1(NULL, PG_Rect(300,140,180,25), "CheckButton 1");
+	PG_PopupMenu popmenu(NULL, 425, 140, "My Menu");
+	PG_PopupMenu submenu(NULL, 425, 140, "My SubMenu");
+	PG_PopupMenu subsubmenu(NULL, 425, 140, "");
 	
 	submenu.sigSelectMenuItem.connect(slot(handle_menu_click), NULL);
 	subsubmenu.sigSelectMenuItem.connect(slot(handle_menu_click), NULL);
@@ -423,7 +425,7 @@ int main(int argc, char* argv[]) {
 
 	check1.Show();
 
-	PG_ScrollBar scroll(NULL, 10, PG_Rect(50, 200, 300, 20), PG_ScrollBar::HORIZONTAL );
+	PG_ScrollBar scroll(NULL, PG_Rect(50, 200, 300, 20), PG_ScrollBar::HORIZONTAL );
 	scroll.Show();
 
 	PG_MaskEdit edit(NULL, PG_Rect(260,0,200,25));
@@ -436,24 +438,25 @@ int main(int argc, char* argv[]) {
 	slider_label.Show();
 
 	// create the slider
-	PG_Slider slider(NULL, 11, PG_Rect(50, 250, 300,20), PG_ScrollBar::HORIZONTAL);
-	slider.SetRange(5,20);
-	//slider.SetTransparency(128);
+	PG_Slider slider(NULL, PG_Rect(50, 250, 300,20), PG_ScrollBar::HORIZONTAL);
+	slider.SetRange(5,50);
 
 	// connect the "MSG_SLIDE" event with "handler_slider" of slider_label
 	slider.sigSlide.connect(slot(slider_label, &MySliderLabel::handler_slider));
+	slider.sigSlideEnd.connect(slot(slider_label, &MySliderLabel::handler_slider));
 
 	slider.Show();
 
-	PG_Button popbtn(NULL, 20, PG_Rect(430, 250,100,25), "Pop me");
+	PG_Button popbtn(NULL, PG_Rect(430, 250,100,25), "Pop me");
 	popbtn.sigClick.connect(slot(handle_popup), (PG_Pointer)&popmenu);
 	popbtn.Show();
 
 	PG_SpinnerBox spin(NULL, PG_Rect(550,250,130,25));
 	spin.Show();
 
-	PG_DropDown drop(NULL, 14, PG_Rect(50, 280, 300,25));
-	drop.SetIndent(5);
+	PG_DropDown drop(NULL, PG_Rect(50, 280, 300,25));
+	
+	//<obsolete>
 	drop.AddItem("Under construction");
 	drop.AddItem("Item 1");
 	drop.AddItem("Item 2");
@@ -461,20 +464,29 @@ int main(int argc, char* argv[]) {
 	drop.AddItem("Item 4");
 	drop.AddItem("Item 5");
 	drop.AddItem("Item 6");
+	//</obsolete>
+	// creating static objects and inserting them into the list
+	// isn't a good idea (the list will try to delete the objects
+	// and the static objects will be destroyed when leaving the
+	// application). Let's use dynamic objects.
+	new PG_ListBoxItem(&drop, 25, "Item 7");
+	new PG_ListBoxItem(&drop, 25, "Item 8");
+	new PG_ListBoxItem(&drop, 25, "Item 9");
+	drop.SetIndent(5);
+
+	slider.sigSlide.connect(slot(hande_slideIndent), (PG_Pointer)&drop);
+	slider.sigSlideEnd.connect(slot(hande_slideIndent), (PG_Pointer)&drop);
 	drop.Show();
 
-	PG_Button list(NULL, BTN_ID_OK, PG_Rect(400,450,100,30), "List");
+	PG_Button list(NULL, PG_Rect(400,450,100,30), "List", PG_Button::OK);
 	list.sigClick.connect(slot(handle_list));
 	list.Show();
 
-	PG_Button quit(NULL, BTN_ID_CLOSE, PG_Rect(600,450,100,30), "Quit");
+	PG_Button quit(NULL, PG_Rect(600,450,100,30), "Quit", PG_Button::CLOSE);
 	quit.sigClick.connect(slot(handle_exit), (PG_Pointer)&app);
 	quit.Show();
 
-	// hehe, now it gets interesting ...
-	PG_Button show_wnd(NULL, BTN_ID_APPLY, PG_Rect(500,450,100,30), "Window");
-
-	// yeah, man.. believe your eyes
+	PG_Button show_wnd(NULL, PG_Rect(500,450,100,30), "Window", PG_Button::APPLY);
 	show_wnd.sigClick.connect(slot(wnd, &TestWindow::handle_show_window));
     
 	show_wnd.Show();
